<?php

namespace App\CrudComponents\Journal;

use App\Components\StaticContentComponent;
use App\CrudComponents\BaseCrudComponent;
use App\CrudComponents\PublicationGroup\PublicationGroupForm;
use App\Model\PublicationGroup;
use App\Model\Submitter;
use Nette\Application\AbortException;
use Nette\Application\UI\InvalidLinkException;
use Nette\ComponentModel\IContainer;
use Nette\Security\User;


class PublicationGroupCrud extends BaseCrudComponent {

	/** @var User */
	protected $loggedUser;

	/** @var PublicationGroup */
	protected $publicationGroupModel;

	/** @var Submitter */
	protected $submitterModel;


    /**
     * JournalCrud constructor.
     * @param User $loggedUser
     * @param PublicationGroup $publicationGroupModel
     * @param Submitter $submitterModel
     * @param IContainer|null $parent
     * @param string|null $name
     */
	public function __construct(
		User $loggedUser,
        PublicationGroup $publicationGroupModel,
		Submitter $submitterModel,
		IContainer $parent = NULL,
        string $name = NULL)
    {
        parent::__construct();
        if ($parent) {
            $parent->addComponent($this, $name);
        }

		$this->addDefaultTemplateVars([
			'publicationGroupAdded'      =>  false,
			'publicationGroupEdited'     =>  false,
			'publicationGroupDeleted'    =>  false,
			'publicationsRelatedToJournal'    =>  [],
		]);

        $this->loggedUser = $loggedUser;
        $this->publicationGroupModel = $publicationGroupModel;
        $this->submitterModel = $submitterModel;
	}

    /**
     * @param string $name
     * @return PublicationGroupForm
     */
	public function createComponentPublicationGroupForm(string $name): PublicationGroupForm
    {
        $form = new PublicationGroupForm($this->publicationGroupModel, $this, $name);

        $form->onError[] = function(){
            $this->redrawControl('publicationGroupForm');
        };

        $form->onSuccess[] = function (PublicationGroupForm $form) {
		    $formValues = $form->getValuesTransformed();
		    $formValues['submitter_id'] = $this->loggedUser->id;

            $checkedSubmitters = explode(',', $formValues['submitters']);
            unset($formValues['submitters']);

		    if (empty($formValues['id'])) {
		        unset($formValues['id']);
			    $record = $this->publicationGroupModel->insert($formValues);
                $this->publicationGroupModel->setGroupSubmitters($record->id, $checkedSubmitters, $this->loggedUser->id);
			    $this->template->publicationGroupAdded = true;
			    $this->onAdd($record);
		    } else {
			    $this->publicationGroupModel->update($formValues);
			    $record = $this->publicationGroupModel->find($formValues['id']);
                $this->publicationGroupModel->setGroupSubmitters($record->id, $checkedSubmitters, $this->loggedUser->id);
			    $this->template->publicationGroupEdited = true;
			    $this->onEdit($record);
		    }

		    if($record) {
			    if ($this->presenter->isAjax()) {
				    $form->clearValues();
				    $this->redrawControl('publicationGroupForm');
			    } else {
				    $this->redirect('this');
			    }
		    }
        };

        return $form;
	}

    /**
     * @param int $id
     * @throws AbortException
     */
	public function handleDelete(int $id): void
    {
		$record = $this->publicationGroupModel->find($id);

		if ($record) {
			$record->toArray(); // load the object to be passed to the callback
			$this->publicationGroupModel->deleteAssociatedRecords($id);
			$this->template->publicationGroupDeleted = true;

			if (!$this->presenter->isAjax()) {
				$this->presenter->redirect('this');
			} else {
				$this->redrawControl('deletePublicationGroup');
			}

			$this->onDelete($record);
		}
	}

    /**
     * @param int $id
     * @throws AbortException
     */
	public function handleEdit(int $id): void
    {
		$publicationGroup = $this->publicationGroupModel->find($id)->toArray();
        $submitters = $this->publicationGroupModel->getGroupSubmitters($id)->fetchPairs(null, 'submitter_id');

        $publicationGroup['submitters'] = join(',', $submitters);

        $this->template->submittersA = $this->publicationGroupModel->getEligibleSubmittersWhoArentInGroup($id)->fetchAssoc('id');
        $this->template->submittersB = $this->submitterModel->findAllBy(['id' => $submitters])->fetchAssoc('id');

        // set up new values
		$this['publicationGroupForm']->setDefaults($publicationGroup);

		if (!$this->presenter->isAjax()) {
			$this->presenter->redirect('this');
		} else {
			$this->redrawControl('publicationGroupForm');
		}

	}

    /**
     * @throws AbortException
     */
	public function handleAdd(): void
    {
        $this->template->submittersA = $this->publicationGroupModel->getEligibleSubmitters()->fetchAssoc('id');

		if (!$this->presenter->isAjax()) {
			$this->presenter->redirect('this');
		} else {
			$this->redrawControl('publicationGroupForm');
		}
	}

    /**
     * @return StaticContentComponent
     * @throws InvalidLinkException
     */
	public function createComponentAddButton(): StaticContentComponent
    {
		$sc = parent::createComponentAddButton();
		$sc->template->addLink =  $this->link('add!');
		return $sc;
	}

    /**
     * @param array|null $params
     */
	public function render(?array $params = []): void
    {
		$this->template->publicationGroupForm = $this['publicationGroupForm'];
		$this->template->getLatte()->addProvider('formsStack', [$this['publicationGroupForm']]);
		parent::render($params);
	}
}
