<?php

namespace App\CrudComponents\Reference;

use App\CrudComponents\BaseCrudComponent;
use App\Model\Publication;
use App\Model\Reference;
use Nette\Application\AbortException;
use Nette\Application\UI\Multiplier;
use \App\CrudComponents\BaseCrudControlsComponent;
use \App\Helpers\ReferenceParser;
use Nette\ComponentModel\IContainer;
use Nette\Security\User;


class ReferenceCrud extends BaseCrudComponent {

	/** @var User */
	protected $loggedUser;

	/** @var  Reference */
	protected $referenceModel;

	/** @var  Publication */
	protected $publicationModel;

	/** @var int */
	protected $publicationId;

	/** @var  array */
	protected $actionsAllowed = ['edit', 'delete', 'add', 'empty'];

	/** @var array - actions that results in changing model */
	protected $writeActions = ['edit', 'delete', 'add', 'empty'];

    /** @var */
    public $onEmpty;


    /**
     * ReferenceCrud constructor.
     * @param int $publicationId
     * @param User $loggedUser
     * @param Publication $publicationModel
     * @param Reference $referenceModel
     * @param IContainer|NULL $parent
     * @param null $name
     */
	public function __construct(
		int $publicationId,
		User $loggedUser,
        Publication $publicationModel,
        Reference $referenceModel,
		IContainer $parent = NULL,
        $name = NULL)
    {
        parent::__construct();
        $parent->addComponent($this, $name);

		$this->addDefaultTemplateVars([
			'entityAdded'   =>  false,
			'entityEdited'  =>  false,
			'entityDeleted' =>  false,
            'entityEmptied' =>  false
		]);

		$this->publicationId = $publicationId;
		$this->referenceModel = $referenceModel;
		$this->publicationModel = $publicationModel;
		$this->loggedUser = $loggedUser;

		$this->onControlsCreate[] = function (BaseCrudControlsComponent &$controlsComponent) {
		    $controlsComponent->addActionAvailable('empty');
		};
	}

    /**
     * @param string $name
     * @return ReferenceAddForm
     */
	public function createComponentAddForm(string $name): ReferenceAddForm
    {
		$form = new ReferenceAddForm($this->publicationId, $this->publicationModel, $this, $name);

		$form->onError[] = function () {
			$this->redrawControl('addForm');
		};

		$form->onSuccess[] = function (ReferenceAddForm $form) {
		    if (!$this->isActionAllowed('add')) {
		        return;
            }

            $formValues = $form->getValuesTransformed();
		    $formValues['submitter_id'] = $this->loggedUser->id;
		    $formValues['publication_id'] = $this->publicationId;
		    $record = $this->referenceModel->insert($formValues);

		    if ($record) {
		        $this->template->entityAdded = true;

		        if ($this->presenter->isAjax()) {
		            $form->clearValues();
		            $this->redrawControl('addForm');
		        } else {
		            $this->redirect('this');
                }

                $this->onAdd($record);
		    }
		};

		return $form;
    }

    /**
     * @param string $name
     * @return ReferenceEditForm
     */
    public function createComponentEditForm(string $name): ReferenceEditForm
    {
        $form = new ReferenceEditForm($this->publicationId, $this->publicationModel, $this, $name);

        $form->onError[] = function () {
            $this->redrawControl('editForm');
        };

        $form->onSuccess[] = function (ReferenceEditForm $form) {
            $formValues = $form->getValuesTransformed();
            $record = $this->referenceModel->find($formValues['id']);

            // reparse if text was changed
            if ($record->text != $formValues['text']) {
                $parser = new ReferenceParser($formValues['text']);
                $parser->parse();
                $formValues['title'] = $parser->getTitle();
            }

            if ($formValues['confirmed'] == 0) {
                $formValues['reference_id'] = null;
            }

            $record = $this->referenceModel->update($formValues);

            if ($record) {
                $this->template->entityEdited = true;

                if ($this->presenter->isAjax()) {
                    $form->clearValues();
                    $this->redrawControl('editForm');
                } else {
                    $this->redirect('this');
                }

                $this->onEdit($record);
            }
        };

        return $form;
    }

    /**
     * @param int $reference_id
     * @throws AbortException
     */
    public function handleDelete(int $reference_id): void
    {
		if (!$this->isActionAllowed('delete')) {
		    return;
        }

		$record = $this->referenceModel->find($reference_id);

		if($record) {
			$record->toArray(); // load the object to be passed to the callback
			$this->referenceModel->delete($reference_id);
			$this->template->entityDeleted = true;

			if (!$this->presenter->isAjax()) {
				$this->presenter->redirect('this');
			} else {
				$this->redrawControl('deleteEntity');
			}

			$this->onDelete($record);
		}
	}

    /**
     * @param int $reference_id
     * @throws AbortException
     */
    public function handleEmpty(int $reference_id): void
    {
        if (!$this->isActionAllowed('empty')) {
            return;
        }

        $record = $this->referenceModel->find($reference_id);

        if ($record) {
            $this->referenceModel->makeEmpty($reference_id);
            $this->template->entityEmptied = true;

            if (!$this->presenter->isAjax()) {
                $this->presenter->redirect('this');
            } else {
                $this->redrawControl('emptyEntity');
            }

            $this->onEmpty($record);
        }
    }

    /**
     * @param int $reference_id
     * @throws AbortException
     */
	public function handleEdit(int $reference_id): void
    {
        $reference = $this->referenceModel->find($reference_id);
        $this['editForm']->setDefaults($reference); // set up new values

        if (!$this->presenter->isAjax()) {
            $this->presenter->redirect('this');
        } else {
            $this->redrawControl('editForm');
        }
    }

    /**
     * @return Multiplier
     */
    public function createComponentControls(): Multiplier
    {
        $templateFile = dirname($this->getReflection()->getFileName()) . '/controls.latte';

        $callbacks = $this->onControlsCreate;

        $lastId = $this->referenceModel->getLastReferenceIdForPublication($this->publicationId);

        return new Multiplier(function ($recordId) use ($templateFile, $callbacks, $lastId) {
            $c = new BaseCrudControlsComponent($recordId, $this->getUniqueId(), $templateFile);

            // fill controls template with actions allowed
            $tmp = new \stdClass();
            $this->fillTemplateWithAllowedActions($tmp);

            // allow deleting only the last reference
            if ($recordId != $lastId) {
                unset($tmp->deleteAllowed);
            }

            $c->addTemplateVars(get_object_vars($tmp));

            foreach ($callbacks as $callback) {
                $callback($c);
            }

            return $c;
        });
    }

}
